#include "dialog.h"
#include "ui_dialog.h"
#include <iostream>
//#include <QPixmap>
//#include <QApplication>
#include <QScreen>
#include <QMessageBox>

Dialog::Dialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::Dialog)
{
    ui->setupUi(this);

    manager = new QNetworkAccessManager(this);

    //connect(manager, SIGNAL(finished(QNetworkReply*)), this, SLOT(addFeedback(QNetworkReply*)));

    QString title = ui->label->text();
    int exitCode = -1;
    QProcess process;
    #ifdef Q_OS_WIN
        qDebug() << "windows";
        process.start("ver");
        process.waitForFinished();
        exitCode = process.exitCode();

        //exitCode = QProcess::execute("systeminfo");
        //exitCode = process.execute("ver");
    #else
        qDebug() << "linux";
        /*
        process.start("lsb_release -a | grep Description | awk '{print $2}'");
        process.waitForFinished();
        exitCode = process.exitCode();

        exitCode = QProcess::execute("lsb_release -a | grep Description | awk '{print $2}'");
        exitCode = process.execute("lsb_release -a | grep Description | awk '{print $2}'");
        */
        QProcess process1;
        process1.setStandardOutputProcess(&process);

        process1.start("lsb_release -a");
        process1.waitForFinished();

        process.start("grep Description");
        process.waitForFinished();

        exitCode = process.exitCode();
    #endif

    qDebug() << exitCode;
    if(exitCode == 0)
    {
        QString OSVersion = QString(process.readAllStandardOutput());
        int index = OSVersion.indexOf(':');
        OSVersion = OSVersion.mid(index + 1).trimmed();
        index = OSVersion.indexOf(' ');
        title = OSVersion.mid(0, index) + title;

        ui->label->setText(title);
    }
    ui->progressBar->hide();
}

Dialog::~Dialog()
{
    delete ui;
}

void Dialog::on_pushButton_clicked()
{

    ui->progressBar->setMaximum(100);
    ui->progressBar->setValue(30);
//    ui->progressBar->show();

    //ui->textEdit->setReadOnly(true);
    ui->textEdit->setDisabled(true);
    ui->lineEdit->setDisabled(true);
    ui->checkBox->setDisabled(true);

    QString description = ui->textEdit->document()->toPlainText();
    if((description == init_description) || (description == ""))
    {
        QMessageBox::critical(0, QString("问题描述为空"), QString("请详细描述您遇到的问题，以便得到快速反馈。"), QString("确定"));
        ui->textEdit->setDisabled(false);
        ui->lineEdit->setDisabled(false);
        ui->checkBox->setDisabled(false);
        return;
    }


    QString email = ui->lineEdit->text();

    //提交反馈
    QNetworkRequest req;
    //req.setHeader(QNetworkRequest::ContentTypeHeader, "application/octet-stream");
    req.setHeader(QNetworkRequest::ContentTypeHeader, "application/x-www-form-urlencoded");

    QUrl url(QString("%1").arg(base_url));
    req.setUrl(url);
    //QByteArray append((const char*)("description=" + description + "&email=" + email).utf16());
    //QByteArray append(("description=" + description + "&email=" + email + "&screenshot=" + screenshot).toLatin1());
    QByteArray append("description=" + QUrl::toPercentEncoding(description.toUtf8()));
    append.append("&email=" + QUrl::toPercentEncoding(email.toUtf8()));
    //append.append("&screenshot=" + QUrl::toPercentEncoding(screenshot));
    append.append("&screenshot=" + QUrl::toPercentEncoding(screenshot));
    qDebug() << append.size();
    //manager->post(QNetworkRequest(url), append);
    reply = manager->post(req, append);
    connect(reply, SIGNAL(readyRead()), this, SLOT(httpReadyRead()));
    connect(reply, SIGNAL(downloadProgress(qint64,qint64)), this, SLOT(updateDataReadProgress(qint64,qint64)));
    connect(reply, SIGNAL(finished()), this, SLOT(httpFinished()));
}

void Dialog::addFeedback(QNetworkReply* reply)
{
    if(reply->error() == QNetworkReply::NoError)
    {
        QByteArray bytes = reply->readAll();
        QString result = QString::fromUtf8(bytes);

        //std::cout << "提交反馈" << result.toStdString() << "!" << std::endl;
        //qDebug() << "提交反馈" << result << "!";
        qDebug() << "提交反馈" + result + "!";
        qDebug() << "=============================";
    }
    else
    {

    }
}

void Dialog::httpReadyRead()
{
    addFeedback(reply);
}

void Dialog::updateDataReadProgress(qint64 bytesRead, qint64 totalBytes)
{
    ui->progressBar->setMaximum(totalBytes);
    ui->progressBar->setValue(bytesRead);
}

void Dialog::httpFinished()
{
    QVariant statusCodeV = reply->attribute(QNetworkRequest::HttpStatusCodeAttribute);
    qDebug() << statusCodeV;
    QVariant redirectionTargetUrl = reply->attribute(QNetworkRequest::RedirectionTargetAttribute);
    qDebug() << redirectionTargetUrl;
    qDebug() << reply->request().url();

    ui->progressBar->hide();
    reply->deleteLater();
    reply =0;
    //删除截图文件
    remove(qPrintable(fileName));

    if(statusCodeV.isValid() && statusCodeV.toInt() < 400)
    {
        /*
        int ok = QMessageBox::warning(this, QString("提交反馈"), QString("你确定提交此次反馈吗？"), QString("确定"), QString("取消"));
        if(ok == 0)
        {
        }
        */
        QMessageBox::information(this, QString("提交反馈"), QString("你的反馈已经成功提交！"), QString("确定"));
        //关闭对话框
        accept();
    }
    else
    {
        QMessageBox::critical(0, QString("提交失败"), QString("请求服务无响应，请稍后再试！"), QString("确定"));
        ui->textEdit->setDisabled(false);
        ui->lineEdit->setDisabled(false);
        ui->checkBox->setDisabled(false);
    }
}

void Dialog::on_checkBox_stateChanged(int arg1)
{
    qDebug() << arg1;
    //截屏并保存为 screenshot.jpg
    fileName = "screenshot.jpg";
    screenshot = "";
    if(ui->checkBox->checkState() == Qt::Checked)
    {
        //std::cout << "截取当前屏幕！" << std::endl;
        qDebug() << "截取当前屏幕！";

        //QPixmap::grabWindow(QApplication::desktop()->winId()).save(fileName,"jpg");
        /****************************************************************
        要注意的是 QApplication::desktop()->winId()，返回的是
        要截屏的窗口的ID ， 设为0 表示当前桌面 ，截屏时就会截全屏
        ******************************************************************/

        //功能同样是截屏 ， 注：QScreen的构造函数是私有的
        this->hide();
        QTime t;
        t.start();
        while (t.elapsed() < 250) {
            QCoreApplication::processEvents();
        }
        QScreen *screen = QGuiApplication::primaryScreen();
        screen->grabWindow(0).save(fileName,"jpg");
        this->show();

        //显示截图
        QImage image(fileName);
        qDebug() << image;
        QImage *imgScaled = new QImage;
        *imgScaled = image.scaled(230, 130, Qt::KeepAspectRatio);
        qDebug() << *imgScaled;
        //QPixmap pixmap(fileName);
        //ui->label_8->setPixmap(pixmap);
        ui->label_7->setPixmap(QPixmap::fromImage(*imgScaled));
        //ui->label_8->show();


        QFile *file = new QFile(fileName);
        if(file->open(QIODevice::ReadOnly))
        {
            //screenshot = file->readAll();
            while(!file->atEnd())
            {
                QByteArray line = file->readLine();
                //screenshot += line;
                screenshot.append(line.toBase64());
            }
            qDebug() << screenshot;
            //qDebug() << QUrl::toPercentEncoding(screenshot);
        }
        file->close();
        delete file;
        file = 0;

        //qDebug() << image.bits();
        //qDebug() << image.byteCount();
        //qDebug() << screenshot.size();
    }
    else
    {
        //删除截图文件
        remove(qPrintable(fileName));
        //不显示截图
        ui->label_7->clear();
        //删除控件
        //ui->label_7->deleteLater();
    }
}

void Dialog::on_textEdit_cursorPositionChanged()
{
    QString description = ui->textEdit->document()->toPlainText();
    if(description == init_description)
    {
        ui->textEdit->document()->clear();
    }
}
